"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMessage = void 0;
const declarations_js_1 = require("./declarations.js");
const names_js_1 = require("./names.js");
const placeholder_js_1 = require("./placeholder.js");
const util_js_1 = require("./util.js");
const values_js_1 = require("./values.js");
// Message ::= Declaration* ( Pattern | Selector Variant+ )
// Selector ::= 'match' ( '{' Expression '}' )+
/**
 * Parse the string syntax representation of a message into
 * its corresponding data model representation.
 *
 * @beta
 */
function parseMessage(src) {
    const errors = [];
    const { declarations, end: pos } = (0, declarations_js_1.parseDeclarations)(src, errors);
    if (src.startsWith('match', pos)) {
        return parseSelectMessage(src, pos, declarations, errors);
    }
    else if (src[pos] === '{') {
        return parsePatternMessage(src, pos, declarations, errors);
    }
    else {
        errors.push({ type: 'parse-error', start: pos, end: src.length });
        return { type: 'junk', declarations, errors, source: src };
    }
}
exports.parseMessage = parseMessage;
function parsePatternMessage(src, start, declarations, errors) {
    const pattern = parsePattern(src, start, errors);
    let pos = pattern.end;
    pos += (0, util_js_1.whitespaces)(src, pos);
    if (pos < src.length) {
        errors.push({ type: 'extra-content', start: pos, end: src.length });
    }
    return { type: 'message', declarations, pattern, errors };
}
function parseSelectMessage(src, start, declarations, errors) {
    let pos = start + 5; // 'match'
    pos += (0, util_js_1.whitespaces)(src, pos);
    const selectors = [];
    while (src[pos] === '{') {
        const ph = (0, placeholder_js_1.parsePlaceholder)(src, pos, errors);
        switch (ph.body.type) {
            case 'expression':
            case 'literal':
            case 'variable':
                break;
            default: {
                const { start, end } = ph.body;
                errors.push({ type: 'bad-selector', start, end });
            }
        }
        selectors.push(ph);
        pos = ph.end;
        pos += (0, util_js_1.whitespaces)(src, pos);
    }
    if (selectors.length === 0) {
        errors.push({ type: 'empty-token', start: pos });
    }
    const variants = [];
    pos += (0, util_js_1.whitespaces)(src, pos);
    while (src.startsWith('when', pos)) {
        const variant = parseVariant(src, pos, selectors.length, errors);
        variants.push(variant);
        pos = variant.end;
        pos += (0, util_js_1.whitespaces)(src, pos);
    }
    if (pos < src.length) {
        errors.push({ type: 'extra-content', start: pos, end: src.length });
    }
    return { type: 'select', declarations, selectors, variants, errors };
}
// Variant ::= 'when' ( WhiteSpace VariantKey )+ Pattern
// VariantKey ::= Literal | Nmtoken | '*'
function parseVariant(src, start, selCount, errors) {
    let pos = start + 4; // 'when'
    const keys = [];
    while (pos < src.length) {
        const ws = (0, util_js_1.whitespaces)(src, pos);
        pos += ws;
        const ch = src[pos];
        if (ch === '{')
            break;
        if (ws === 0) {
            errors.push({ type: 'missing-char', char: ' ', start: pos });
        }
        let key;
        switch (ch) {
            case '*':
                key = { type: '*', start: pos, end: pos + 1 };
                break;
            case '(':
                key = (0, values_js_1.parseLiteral)(src, pos, errors);
                break;
            default:
                key = (0, names_js_1.parseNmtoken)(src, pos, errors);
        }
        if (key.end === pos)
            break; // error; reported in pattern.errors
        keys.push(key);
        pos = key.end;
    }
    if (selCount > 0 && keys.length !== selCount) {
        const end = keys.length === 0 ? pos : keys[keys.length - 1].end;
        errors.push({ type: 'key-mismatch', start, end });
    }
    const value = parsePattern(src, pos, errors);
    return { start, end: value.end, keys, value };
}
// Pattern ::= '{' (Text | Placeholder)* '}' /* ws: explicit */
function parsePattern(src, start, errors) {
    if (src[start] !== '{') {
        errors.push({ type: 'missing-char', char: '{', start });
        return { start, end: start, body: [] };
    }
    let pos = start + 1;
    const body = [];
    loop: while (pos < src.length) {
        switch (src[pos]) {
            case '{': {
                const ph = (0, placeholder_js_1.parsePlaceholder)(src, pos, errors);
                body.push(ph);
                pos = ph.end;
                break;
            }
            case '}':
                break loop;
            default: {
                const tx = (0, values_js_1.parseText)(src, pos, errors);
                body.push(tx);
                pos = tx.end;
            }
        }
    }
    if (src[pos] === '}') {
        pos += 1;
    }
    else {
        errors.push({ type: 'missing-char', char: '}', start: pos });
    }
    return { start, end: pos, body };
}
