"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parsePlaceholder = void 0;
const names_js_1 = require("./names.js");
const util_js_1 = require("./util.js");
const values_js_1 = require("./values.js");
// Placeholder ::= '{' (Expression | Markup | MarkupEnd)? '}'
function parsePlaceholder(src, start, errors) {
    let pos = start + 1; // '{'
    pos += (0, util_js_1.whitespaces)(src, pos);
    let arg;
    switch (src[pos]) {
        case '(':
            arg = (0, values_js_1.parseLiteral)(src, pos, errors);
            pos = arg.end;
            break;
        case '$': {
            arg = (0, values_js_1.parseVariable)(src, pos, errors);
            pos = arg.end;
            break;
        }
    }
    let body;
    let junkError;
    pos += (0, util_js_1.whitespaces)(src, pos);
    switch (src[pos]) {
        case ':':
        case '+':
        case '-':
            body = parseExpressionOrMarkup(src, pos, arg, errors);
            pos = body.end;
            break;
        default:
            if (arg) {
                body = arg;
            }
            else {
                const source = src.substring(pos, pos + 1);
                body = { type: 'junk', start: pos, end: pos, source };
                junkError = { type: 'parse-error', start: pos, end: pos };
                errors.push(junkError);
            }
    }
    pos += (0, util_js_1.whitespaces)(src, pos);
    if (pos >= src.length) {
        errors.push({ type: 'missing-char', char: '}', start: pos });
    }
    else if (src[pos] !== '}') {
        const errStart = pos;
        while (pos < src.length && src[pos] !== '}')
            pos += 1;
        if (body.type === 'junk') {
            body.end = pos;
            body.source = src.substring(body.start, pos);
            if (junkError)
                junkError.end = pos;
        }
        else {
            errors.push({ type: 'extra-content', start: errStart, end: pos });
        }
    }
    else {
        pos += 1;
    }
    return { type: 'placeholder', start, end: pos, body };
}
exports.parsePlaceholder = parsePlaceholder;
// Expression ::= Operand Annotation? | Annotation
// Operand ::= Literal | Variable
// Annotation ::= Function Option*
// Function ::= ':' Name /* ws: explicit */
// Markup ::= MarkupStart Option*
// MarkupStart ::= '+' Name /* ws: explicit */
// MarkupEnd ::= '-' Name /* ws: explicit */
function parseExpressionOrMarkup(src, start, operand, errors) {
    const sigil = src[start];
    let pos = start + 1; // ':' | '+' | '-'
    const name = (0, names_js_1.parseNameValue)(src, pos);
    if (!name)
        errors.push({ type: 'empty-token', start: pos });
    const options = [];
    pos += name.length;
    while (pos < src.length) {
        const ws = (0, util_js_1.whitespaces)(src, pos);
        if (src[pos + ws] === '}')
            break;
        if (ws === 0)
            errors.push({ type: 'missing-char', char: ' ', start: pos });
        pos += ws;
        const opt = parseOption(src, pos, errors);
        if (opt.end === pos)
            break; // error
        options.push(opt);
        pos = opt.end;
    }
    const end = pos;
    if (sigil === ':') {
        return { type: 'expression', operand, start, end, name, options };
    }
    if (operand) {
        const { start, end } = operand;
        errors.unshift({ type: 'extra-content', start, end });
    }
    if (sigil === '+') {
        return { type: 'markup-start', start, end, name, options };
    }
    if (options.length > 0) {
        errors.push({
            type: 'extra-content',
            start: options[0].start,
            end: options[options.length - 1].end
        });
    }
    return { type: 'markup-end', start, end, name };
}
// Option ::= Name '=' (Literal | Nmtoken | Variable)
function parseOption(src, start, errors) {
    const name = (0, names_js_1.parseNameValue)(src, start);
    let pos = start + name.length;
    pos += (0, util_js_1.whitespaces)(src, pos);
    if (src[pos] === '=')
        pos += 1;
    else
        errors.push({ type: 'missing-char', char: '=', start: pos });
    pos += (0, util_js_1.whitespaces)(src, pos);
    let value;
    switch (src[pos]) {
        case '(':
            value = (0, values_js_1.parseLiteral)(src, pos, errors);
            break;
        case '$':
            value = (0, values_js_1.parseVariable)(src, pos, errors);
            break;
        default:
            value = (0, names_js_1.parseNmtoken)(src, pos, errors);
    }
    return { start, end: value.end, name, value };
}
