"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyMessage = void 0;
const data_model_1 = require("../data-model");
const messageformat_1 = require("../messageformat");
const names_1 = require("../parser/names");
const pattern_1 = require("../pattern");
/**
 * Stringify a message using its syntax representation.
 *
 * @beta
 */
function stringifyMessage(msg) {
    if (msg instanceof messageformat_1.MessageFormat)
        msg = msg.resolvedOptions().message;
    let res = '';
    for (const decl of msg.declarations)
        res += stringifyDeclaration(decl);
    if ((0, data_model_1.isPatternMessage)(msg)) {
        res += stringifyPattern(msg.pattern);
    }
    else if ((0, data_model_1.isSelectMessage)(msg)) {
        res += 'match';
        for (const sel of msg.selectors)
            res += ' ' + stringifyPlaceholder(sel);
        for (const { keys, value } of msg.variants) {
            res += '\nwhen ';
            for (const key of keys) {
                res += ((0, pattern_1.isLiteral)(key) ? stringifyLiteral(key) : '*') + ' ';
            }
            res += stringifyPattern(value);
        }
    }
    else {
        res += msg.source.trim();
    }
    return res;
}
exports.stringifyMessage = stringifyMessage;
function stringifyDeclaration({ target, value }) {
    const targetStr = (0, pattern_1.isVariableRef)(target)
        ? stringifyVariableRef(target)
        : stringifyJunk(target);
    const valueStr = (0, pattern_1.isPlaceholder)(value)
        ? stringifyPlaceholder(value)
        : stringifyJunk(value);
    return `let ${targetStr} = ${valueStr}\n`;
}
function stringifyExpression({ name, operand, options }) {
    let res;
    if ((0, pattern_1.isLiteral)(operand)) {
        res = stringifyLiteral(operand) + ' ';
    }
    else if ((0, pattern_1.isVariableRef)(operand)) {
        res = stringifyVariableRef(operand) + ' ';
    }
    else {
        res = '';
    }
    res += `:${name}`;
    if (options)
        for (const opt of options)
            res += ' ' + stringifyOption(opt);
    return res;
}
function stringifyJunk(junk) {
    return junk.source.trim();
}
function stringifyLiteral(lit) {
    if (lit.type === 'nmtoken' && (0, names_1.isValidNmtoken)(lit.value)) {
        return lit.value;
    }
    const esc = lit.value
        .replace(/\\/g, '\\\\')
        .replace(/\(/g, '\\(')
        .replace(/\)/g, '\\)');
    return `(${esc})`;
}
function stringifyMarkupEnd(end) {
    return `-${end.name}`;
}
function stringifyMarkupStart({ name, options }) {
    let res = `+${name}`;
    if (options)
        for (const opt of options)
            res += ' ' + stringifyOption(opt);
    return res;
}
function stringifyOption(opt) {
    const valueStr = (0, pattern_1.isVariableRef)(opt.value)
        ? stringifyVariableRef(opt.value)
        : stringifyLiteral(opt.value);
    return `${opt.name}=${valueStr}`;
}
function stringifyPattern({ body }) {
    let res = '';
    for (const el of body) {
        res += (0, pattern_1.isText)(el) ? el.value : stringifyPlaceholder(el);
    }
    return `{${res}}`;
}
function stringifyPlaceholder(ph) {
    const body = (0, pattern_1.isPlaceholder)(ph) ? ph.body : ph;
    let res;
    switch (body.type) {
        case 'expression':
            res = stringifyExpression(body);
            break;
        case 'junk':
            res = stringifyJunk(body);
            break;
        case 'literal':
            res = stringifyLiteral(body);
            break;
        case 'markup-end':
            res = stringifyMarkupEnd(body);
            break;
        case 'markup-start':
            res = stringifyMarkupStart(body);
            break;
        case 'variable':
            res = stringifyVariableRef(body);
            break;
        default:
            res = ''; // bad placeholder
    }
    return `{${res}}`;
}
function stringifyVariableRef(ref) {
    return '$' + ref.name;
}
